/*
**this simple rootscript example shows how to develop  a script that will
**do the following:
**
** 1)configure Root-2 in auto mode, apply power, reset the device (device will enumerate automatically)
**
** 2)send 1024 bytes to the device in 64 byte packets, 8 per frame (microframe if high speed)
**
** This example assumes that a device is attached to root2, with 
** a BULK OUT endpoint at endpoint 2, capable of sinking 64 byte packets at
** the specified rate.
**
**
**
**
**
**
**
*/

/*** Includes ***/
#include <stdio.h>
#include <stdlib.h>

#include "stddefs.h"

#include "rootscript.h"


//this example will run at either full or high speed.
//choose which you want by commenting out one or the other below...

#define TEST_SPEED DT_FULL_SPEED
//#define TEST_SPEED DT_HIGH_SPEED



//the structure below defines most of the parameters for the block command.
//you can change these fields to play with the results.....

const BLOCK_DEV_TRANS_STRUCT default_block_dev_transH = 
{
	2,						//	UBYTE Address;
	2,						//	UBYTE Endpoint;
	OUT_PID,				//	UBYTE Pid;
	DATA0_PID,				//	UBYTE DataPid;
	TEST_SPEED | DT_STOP_ON_NAK,			//  UWORD Control;	
	1,						//  UWORD ServiceInterval;
	8,						//  UBYTE PacketsPerFrame;
	64,						//  UWORD Packetsize;
	(UBYTE *) 0,			//  UBYTE *DataOut;
	0						//	ULONG Datalen;
};

/* Local Function Declarations */
void GenDelay(ULONG mS);
void FillTestBuffer(void);


//invent a test buffer to hold our test data......
#define TEST_BUFFER_SIZE 1024
UBYTE test_data[TEST_BUFFER_SIZE];



/* Local Statics */
ULONG ScriptBytes;
FILE *usb_stream;



int main(int argc, char **argv)
{
	char *usbfile;
	BLOCK_DEV_TRANS_STRUCT testH;

	if (argc != 2) {
		printf("\n*** Usage: testscript <usb_filename>\n");
		exit (1);
	}

	usbfile = argv[1];

	/** Open File for writing the rootscript **/

	printf ("USB File: %s\n",usbfile);

	usb_stream = RS_Fopen(usbfile) ;
	if (usb_stream == NULL)
	{
		printf ("*** Can't open USB output file %s\n",usbfile);
		exit(1);
	}



	/*** Initialize USB program file ***/

	RS_Program(usb_stream);	/* write program command */
	ScriptBytes = 0;		/* init script byte count */

/*--- Start of Script Body ---*/
	ScriptBytes += RS_Response(usb_stream,QUIET);	//set quiet mode

	//depending on speed of our test device, set up root2 to allow or inhibit
	//high speed connects.
	if (TEST_SPEED == DT_FULL_SPEED)
		ScriptBytes += RS_RootConfig(usb_stream,
		ROOT1_CONNECT_SPEED_CONTROL,INHIBIT_HS);	//set to inhibit high speed
	else
		ScriptBytes += RS_RootConfig(usb_stream,
		ROOT1_CONNECT_SPEED_CONTROL,ALLOW_HS);		//set to allow high speed

	//
	ScriptBytes += RS_RootConfig(usb_stream,ROOT1_MODE,AUTO_MODE);	//turn on auto mode
	ScriptBytes += RS_Power(usb_stream,1);							//turn on power
	GenDelay(1000);													//delay 1 second
	ScriptBytes += RS_USB_Reset(usb_stream);						// Reset device 
	GenDelay(1000);													// delay 1 second 

	//because root1 is running in auto mode, the above reset will cause Root2 to enumerate
	//and configure the device.  The 1 second delay is adequate to allow this to happen.

	ScriptBytes += RS_RootConfig(usb_stream,ROOT1_MODE,MANUAL_MODE); //flip back to manual mode

	//by turning Root2 back to manual mode, we can avoid any bus traffic that root2 will
	//otherwise automatically generate.  This can only be endpoint polling to interrupt IN
	//endpoints, but it never hurts to go to manual mode anyway....

	FillTestBuffer();												//fill the test buffer with
																	//known data.
	testH = default_block_dev_transH;								//initialize our test structure
	testH.DataOut = test_data;										//aim at test data
	testH.Datalen = TEST_BUFFER_SIZE;								//aim at length of test data
	ScriptBytes += RS_BlockDevTrans (usb_stream,&testH);			//do the transfer
	GenDelay(1000);													//delay one second
	ScriptBytes += RS_End(usb_stream);								//end the script


	printf ("\nComplete...\n RS_End at Index %u\n Total Byte Count = %u\n", RS_CurIdx() - 1, ScriptBytes);
	
	RS_Fclose(usb_stream);
	return(1);
	exit(0);

}



/*** GenDelay()
*	Generate code to produce a time delay
***/
void GenDelay(ULONG mS) {	/* generate a delay of mS mS */
	UWORD i;
	ScriptBytes += RS_Timer(usb_stream,mS);			// set timer for delay 
	i = RS_CurIdx() + 2;							// i = index of next script command after delay
	ScriptBytes += RS_Cond(usb_stream, CHK_Timeout, 
		i, ENABLED);								// Enable timer to index following RS_Check 
	ScriptBytes += RS_Check(usb_stream,0);			// delay until timer expires 
	return;
}


void FillTestBuffer(void)
{
	int i;
	for (i = 0; i < TEST_BUFFER_SIZE; i++)
		test_data[i] = i;
}
